import numpy as np
import matplotlib.pyplot as plt
from control import tf, feedback, bode_plot, step_response

def thickBode(sys, fignum):
    omega = np.logspace(-1, 2, 500)
    mag, phase, omega = bode_plot(sys, omega=omega, Hz=False, dB=False, wrap_phase=True, plot=False)
    fout = omega / (2 * np.pi)
    phase = phase * 180 / np.pi

    plt.figure(fignum)
    plt.loglog(fout, mag.squeeze(), linewidth=3)
    plt.grid(True)
    plt.title("Magnitude versus frequency (hertz) (log-log scale)")
    plt.xlabel("Frequency in Cycles per second")
    plt.ylabel("Magnitude")

    plt.figure(fignum + 1)
    plt.semilogx(fout, phase.squeeze(), linewidth=3)
    plt.grid(True)
    plt.title("Phase (degrees) versus frequency (hertz) (log-linear scale)")
    plt.xlabel("Frequency in Cycles per second")
    plt.ylabel("Phase in Degrees")

# Define transfer function variable
s = tf('s')

# Controller Gains and transfer function
Kp = 0.5
Kd = 0.15
Ki = 4

Ke2c = Kp + Kd * s + Ki / s  # Error to motor command

# Motor Command to angular acceleration
# Adjust Beta and Ga so that your arm's response to sines match
Ga = 10 # Change to match model
Beta = -10 # Change to match model
Hc2a = 1 # FIX! 

# Angular acceleration to angle transfer function
Ha2t = 1 # FIX!

# Determine closed loop transfer function
Gs = feedback(Ke2c*Hc2a*Ha2t,1);  

# Plots (1) Bode (2,3) Mag, Phase versus freq (hertz) (4) Step response
plt.figure(1)
bode_plot(Gs, wrap_phase=True)  
thickBode(Gs, 2)  # Gives you |Gs| and angle(Gs) vs f (hertz)

# Closed loop step response
plt.figure(4)
t, y = step_response(Gs)
plt.plot(t, y)

# Plot function on linear scale
plt.figure(5)
omega = np.logspace(-1, 2, 500)
mag, phase, omega = bode_plot(Gs, omega=omega, wrap_phase=True, plot=False)
phase = phase * 180 / np.pi

plt.subplot(2, 1, 1)
plt.plot(omega / (2 * np.pi), mag.squeeze())
plt.grid(True)
plt.xlim([0, 5])
plt.xlabel("Frequency (Hz)")
plt.ylabel("Magnitude (abs)")

plt.subplot(2, 1, 2)
plt.plot(omega / (2 * np.pi), phase.squeeze())
plt.grid(True)
plt.xlim([0, 5])
plt.xlabel("Frequency (Hz)")
plt.ylabel("Phase (Degrees)")

plt.show()

